import 'dart:collection';
import 'dart:convert';

import 'package:finotescore/observe.dart';
import 'package:finotescore/src/platform_interface.dart';
import 'package:http/http.dart';
import 'package:http/io_client.dart';
import 'package:http_interceptor/http_interceptor.dart';
import 'package:uuid/uuid.dart';

class NetworkUtils {
  String uuidValue = Uuid().v1();

  void onRequest(Object data) {
    HashMap<String, Object> requestMap = HashMap<String, Object>();
    String? baseUrl = data.runtimeType == RequestData
        ? (data as RequestData).baseUrl
        : data.runtimeType == Request
            ? "${(data as Request).url.origin}:${(data).url.port}/${(data).url.path}"
            : null;
    if (baseUrl == null) return;
    List<String>? whiteListedDomains = Observe().getWhiteListedDomains();
    bool allowDomainWatch = true;
    if (whiteListedDomains != null && whiteListedDomains.isNotEmpty) {
      allowDomainWatch = false;
      for (String whiteListedDomain in whiteListedDomains) {
        if (baseUrl.contains(whiteListedDomain)) allowDomainWatch = true;
      }
    }

    if (!allowDomainWatch) return;

    Map<String, int>? baseUrlTimeouts = Observe().getBaseUrlTimeouts();
    if (baseUrlTimeouts != null && baseUrlTimeouts.isNotEmpty) {
      for (String baseUrlFromList in baseUrlTimeouts.keys) {
        if (baseUrl.contains(baseUrlFromList) ||
            baseUrlFromList.contains(baseUrl)) {
          int? timeout = baseUrlTimeouts[baseUrlFromList];
          if (timeout != null || timeout! < 1000) {
            requestMap["timeout"] = timeout;
            break;
          }
        }
      }
    }

    requestMap["requestUrl"] = baseUrl;
    requestMap["method"] = data.runtimeType == RequestData
        ? methodToString((data as RequestData).method)
        : (data as Request).method;
    dynamic requestBody =
        data.runtimeType == RequestData ? (data as RequestData).body : null;
    if (requestBody != null) {
      requestMap["requestBodySize"] = requestBody.toString().length;
      requestMap["requestBody"] = requestBody.toString();
    }

    requestMap["query"] = json.encode(data.runtimeType == RequestData
        ? (data as RequestData).params
        : (data as Request).url.queryParameters);

    List<String>? headerKeysToBeMasked = Observe().getMaskedHeaderKeys();
    if (headerKeysToBeMasked != null)
      requestMap["headerKeysToBeMasked"] = headerKeysToBeMasked;

    requestMap["requestHeaders"] = data.runtimeType == RequestData
        ? (data as RequestData).headers
        : (data as Request).headers;
    requestMap["id"] = uuidValue;

    PlatformInterface.setAPIRequest(requestMap);
  }

  void onResponse(Object data) {
    HashMap<String, Object> requestMap = HashMap<String, Object>();
    requestMap["statusCode"] = data.runtimeType == ResponseData
        ? (data as ResponseData).statusCode
        : (data as IOStreamedResponse).statusCode;
    dynamic responseBody =
        data.runtimeType == ResponseData ? (data as ResponseData).body : null;
    if (responseBody != null) {
      requestMap["responseBodySize"] = responseBody.toString().length;
      requestMap["responseBody"] = responseBody.toString();
    } else if (data.runtimeType == IOStreamedResponse) {
      int? contentLength = (data as IOStreamedResponse).contentLength;
      if (contentLength != null) requestMap["responseBodySize"] = contentLength;
    }

    if (data.runtimeType == ResponseData &&
        (data as ResponseData).headers != null) {
      requestMap["responseHeaders"] = data.headers!;
    } else if (data.runtimeType == IOStreamedResponse) {
      requestMap["responseHeaders"] = (data as IOStreamedResponse).headers;
    }
    requestMap["id"] = uuidValue;

    PlatformInterface.setAPIResponse(requestMap);
  }
}
